import React from 'react'
import { Dashboard, Layout, Loading } from './components'
import { withRouter } from 'react-router-dom'
import moment from 'moment'

// types
import { types } from '@joystream/types'
import { ApiPromise, WsProvider } from '@polkadot/api'
import { Header } from '@polkadot/types/interfaces'
const wsLocation = 'wss://rome-rpc-endpoint.joystream.org:9944/'

const initialState = { blocks: [], block: 0, loading: true }

class App extends React.Component {
  async initializeSocket() {
    const provider = new WsProvider(wsLocation)
    const api = await ApiPromise.create({ provider, types })
    await api.isReady

    const [chain, node, version] = await Promise.all([
      api.rpc.system.chain(),
      api.rpc.system.name(),
      api.rpc.system.version(),
    ])
    this.setState({ chain, node, version, loading: false })

    api.rpc.chain.subscribeNewHeads(async (header: Header): Promise<void> => {
      const timestamp = moment().valueOf()
      const blocks = this.state.blocks.concat({ ...header, timestamp })
      this.setState({ blocks, block: header.number.toNumber() })
    })
  }

  async fetchData() {
    // initial axios requests go here
    this.setState({ loading: false })
  }

  /** RENDER FUNCTIONS **/
  renderLoading() {
    return <Loading />
  }
  renderError() {
    if (this.state.showModal === 'Error') return
    this.setShowModal('Error')
  }
  renderApp() {
    const { block } = this.state
    const blocks = this.state.blocks
      .sort((a, b) => b.timestamp - a.timestamp)
      .slice(0, 10)
    return (
      <div className="w-100 h-100 d-flex flex-grow-1 align-items-center justify-content-center">
        <Dashboard block={block} blocks={blocks} />
      </div>
    )
  }
  render() {
    if (this.state.loading) return this.renderLoading()
    if (this.state.error) this.renderError()
    if (this.state.component === 'layout') return <Layout {...this.state} />
    return this.renderApp()
  }

  componentDidMount() {
    this.initializeSocket()
    //this.fetchData()
  }
  componentWillUnmount() {
    console.log('unmounting...')
  }
  constructor() {
    super()
    this.state = initialState
  }
}

export default withRouter(App)
